# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/routes/instance_config_mfa.ipynb.

# %% auto 0
__all__ = ['MFA_UPDATE_Error', 'MFA_UPDATE_Value_Error', 'toggle_enable_mfa', 'MFA_GET_Error', 'get_mfa_config',
           'set_mfa_max_code_attempts', 'set_mfa_how_long_valid']

# %% ../../nbs/routes/instance_config_mfa.ipynb 2
import httpx

from typing import List

import domolibrary.client.get_data as gd
import domolibrary.client.ResponseGetData as rgd
import domolibrary.client.DomoAuth as dmda
import domolibrary.client.DomoError as de

# from domolibrary.utils.convert import convert_string_to_bool
from typing import List, Union


# %% ../../nbs/routes/instance_config_mfa.ipynb 7
class MFA_UPDATE_Error(de.DomoError):
    def __init__(
        self,
        res: rgd.ResponseGetData,
        message=None,
    ):

        super().__init__(
            domo_instance=res.auth.domo_instance,
            function_name=res.traceback_details.function_name,
            status=res.status,
            message=message or res.response,
        )


class MFA_UPDATE_Value_Error(de.DomoError):
    def __init__(
        message,
    ):

        super().__init__(
            message=message,
        )


@gd.route_function
async def toggle_enable_mfa(
    auth: dmda.DomoAuth,
    is_enable_MFA: bool = False,
    session: httpx.AsyncClient = None,
    debug_api: bool = False,
    parent_class: str = None,
    debug_num_stacks_to_drop: int = 1,
) -> rgd.ResponseGetData:

    # url = f'https://{auth.domo_instance}.domo.com/api/content/v1/customer-states

    url = f"https://{auth.domo_instance}.domo.com/api/content/v1/customer-states/domo.policy.multifactor.required"

    payload = {
        "name": "domo.policy.multifactor.required",
        "value": "yes" if is_enable_MFA else "no",
    }

    res = await gd.get_data(
        auth=auth,
        url=url,
        method="PUT",
        body=payload,
        debug_api=debug_api,
        session=session,
        parent_class=parent_class,
        num_stacks_to_drop=debug_num_stacks_to_drop,
    )

    if not res.is_success:
        raise MFA_UPDATE_Error(
            res=res,
            message=f"failed to toggle MFA in {auth.domo_instance}",
        )

    res.response = f'toggled MFA {"on" if is_enable_MFA else "off"}'

    return res

# %% ../../nbs/routes/instance_config_mfa.ipynb 10
class MFA_GET_Error(de.DomoError):
    def __init__(
        self,
        res : rgd.ResponseGetData,
        message = None,
    ):

        super().__init__(
            domo_instance=res.auth.domo_instance,
            function_name=res.traceback_details.function_name,
            status=res.status,
            message=message or res.response,
        )
        
@gd.route_function
async def get_mfa_config(
    auth: dmda.DomoAuth,
    
    incl_is_multifactor_required: bool = True,
    incl_how_long_valid: bool = True, # how long MFA is valid
    incl_max_code_attempts: bool = True,
    
    session: httpx.AsyncClient = None,
    parent_class: str = None,
    debug_api: bool = False,
    debug_num_stacks_to_drop: int = 1
) -> rgd.ResponseGetData:

    params = {"ignoreCache" :True}
    
    state_ls = []

    if incl_is_multifactor_required:
        state_ls.append('domo.policy.multifactor.required')

    if incl_how_long_valid:
        state_ls.append('domo.policy.multifactor.factorExpires')
    
    
    if incl_max_code_attempts:
        state_ls.append('domo.policy.multifactor.maxCodeAttempts')
    
    params.update({'stateName' : ",".join(state_ls)})
        
    url = f'https://{auth.domo_instance}.domo.com/api/content/v1/customer-states'
    
    res = await gd.get_data(
        auth=auth,
        url=url,
        method="GET",
        debug_api=debug_api,
        session=session,
        parent_class=parent_class,
        num_stacks_to_drop=debug_num_stacks_to_drop,
        params = params
    )
    
    if not res.is_success:
        raise MFA_GET_Error(
            res=res,
            message=f"failed to retrieve {auth.domo_instance} MFA configuration settings",
        )

    return res


# %% ../../nbs/routes/instance_config_mfa.ipynb 13
@gd.route_function
async def set_mfa_max_code_attempts(
    auth: dmda.DomoAuth,
    num_attempts: int, 
    session: httpx.AsyncClient = None,
    debug_api: bool = False,
    parent_class: str = None,
    debug_num_stacks_to_drop: int = 1
) -> rgd.ResponseGetData:
    
    '''Sets the max number of invalid log in attempts before the MFA code is reset'''
    
    url = f'https://{auth.domo_instance}.domo.com/api/content/v1/customer-states/domo.policy.multifactor.maxCodeAttempts'
    
    if not num_attempts > 0:
        raise MFA_UPDATE_Value_Error(
            message="max num_attempts must be greater than 0. unable to set MFA max code attempts",
        )
        
    payload = {'name': "domo.policy.multifactor.maxCodeAttempts", 'value': num_attempts}
    
    res = await gd.get_data(
        auth=auth,
        url=url,
        method="PUT",
        body=payload,
        debug_api=debug_api,
        session=session,
        parent_class=parent_class,
        num_stacks_to_drop=debug_num_stacks_to_drop
    )
    
    if not res.is_success:
        raise MFA_UPDATE_Error(
            res=res,
            message=f"failed to update max number of code attempts for MFA in {auth.domo_instance}",
        )

    res.response = f"set max number of code attempts to {num_attempts} in {auth.domo_instance}"

    return res

# %% ../../nbs/routes/instance_config_mfa.ipynb 16
@gd.route_function
async def set_mfa_how_long_valid(
    auth: dmda.DomoAuth,
    num_days_valid: int, 
    session: httpx.AsyncClient = None,
    debug_api: bool = False,
    parent_class: str = None,
    debug_num_stacks_to_drop: int = 1
) -> rgd.ResponseGetData:
    '''Sets the number of days before the user must re-authenticate'''
    
    url = f'https://{auth.domo_instance}.domo.com/api/content/v1/customer-states/domo.policy.multifactor.factorExpires'
    
    if not num_days_valid > 0:
        raise MFA_UPDATE_Value_Error(
            message="max num_days must be greater than 0. unable to set days before MFA expires",
        )
        
    payload = {'name': "domo.policy.multifactor.factorExpires", 'value': num_days_valid}
    
    res = await gd.get_data(
        auth=auth,
        url=url,
        method="PUT",
        body=payload,
        debug_api=debug_api,
        session=session,
        parent_class=parent_class,
        num_stacks_to_drop=debug_num_stacks_to_drop
    )
    
    if not res.is_success:
        raise MFA_UPDATE_Error(
            res=res,
            message=f"failed to set number of days before MFA expires in {auth.domo_instance}",
        )

    res.response = f"num days MFA valid set to {num_days_valid} in {auth.domo_instance}"

    return res
